
#include "p18cxxx.h"
#include "CTMU.h"

#define COUNT 2000 //@ 8MHz = 125uS.
#define DELAY for(i=0;i<COUNT;i++)
#define OPENSW 1000 //Un-pressed switch value
#define TRIP 300 //Difference between pressed
//and un-pressed switch
#define HYST 65 //amount to change
//from pressed to un-pressed
#define PRESSED 1
#define UNPRESSED 0

/**************************************************************************/
/*Setup CTMU *****************************************************************/
/**************************************************************************/
void SetupTouch(void)
{
	InitCTMU();
	InitADC(); 
}


void InitCTMU(void)
{
	/**************************************************************************/
	//Setup CTMU;
	/**************************************************************************/
	CTMUICON=0x03;
	CTMUCONHbits.IDISSEN=0;
	CTMUCONHbits.CTTRIG=0;
	CTMUCONHbits.EDGEN=0;
	CTMUCONLbits.EDG1POL=1;
	CTMUCONLbits.EDG2POL=1;
	CTMUCONLbits.EDG1STAT=0;
	CTMUCONLbits.EDG2STAT=0;
	PIR3bits.CTMUIF=0;
	CTMUCONHbits.CTMUEN=1;	
}

void InitADC(void)
{
	/**************************************************************************/
	//Setup AD converter;
	/**************************************************************************/
	TRISAbits.RA0=1; //set channels as an input
	TRISAbits.RA1=1;
	TRISAbits.RA2=1;
	TRISAbits.RA3=1;
	// Configured ANx as an analog channel
	// ANCON0
	ANCON0 = 0XF0;
	// ANCON1
	ANCON1 = 0X1F;
	// ADCON1
	ADCON1bits.ADFM=1; // Resulst format 1= Right justified
	ADCON1bits.ADCAL=0; // Normal A/D conversion operation
	ADCON1bits.ACQT=1; // Acquition time 7 = 20TAD 2 = 4TAD 1=2TAD
	ADCON1bits.ADCS=2; // Clock conversion bits 6= FOSC/64 2=FOSC/32
	ANCON1bits.VBGEN=1; // Turn on the Bandgap needed for Rev A0 parts
	// ADCON0
	ADCON0bits.VCFG0 =0; // Vref+ = AVdd
	ADCON0bits.VCFG1 =0; // Vref- = AVss
	ADCON0bits.CHS=2; // Select ADC channel 2
	ADCON0bits.ADON=1; // Turn on ADC
}

int	ReadFromCTMU(Channel_No)
{
	/* Storage for reading the voltage */
	unsigned int Vread; 

	/* Select ADC channel */
	ADCON0bits.CHS = Channel_No; 

	/* Set the ITRIM value for the callibration of the channel */
	CTMUICONbits.ITRIM = g_mTouchCalibrationITRIM[buttonID];

	/* Enable the CTMU */
	CTMUCONHbits.CTMUEN = 1;

	/* Drain charge on the circuit */
	CTMUCONHbits.IDISSEN = 1; 
	Nop();		Nop();		Nop();		Nop();		Nop();
	Nop();		Nop();		Nop();		Nop();		Nop();
	Nop();		Nop();		Nop();		Nop();		Nop();
	Nop();		Nop();		Nop();		Nop();		Nop();

	/* End drain of circuit */
	CTMUCONHbits.IDISSEN = 0;
 
	/* Begin charging the circuit using CTMU current source */
	CTMUCONLbits.EDG1STAT = 1; 
//	Nop();		Nop();		Nop();		Nop();		Nop();
//	Nop();		Nop();		Nop();		Nop();		Nop();
//	Nop();		Nop();		Nop();		Nop();		Nop();
	Nop();		Nop();		Nop();		Nop();		Nop();
	Nop();		Nop();		Nop();		Nop();		Nop();
	Nop();		Nop();		Nop();		Nop();		Nop();
	Nop();		Nop();		Nop();		Nop();		Nop();

	/* Stop charging circuit */
	CTMUCONLbits.EDG1STAT = 0;

	/* Make sure A/D interrupt is not set */
	PIR1bits.ADIF = 0; 
	
	/* Begin A/D conversion */
	ADCON0bits.GO=1; 
	//Wait for A/D convert complete
	while(!PIR1bits.ADIF);

	/* Get the value from the A/D */
	Vread = ADRES;

	return Vread;

}

